/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "host/atca_host.h"
#include "../../slot_5_secret_key.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
#define SYMMETRIC_KEY_SLOT  (0x05)

extern ATCAIfaceCfg atecc608_0_init_data;
extern uint8_t slot_5_secret_key[];
uint8_t num_in[NONCE_NUMIN_SIZE];
uint8_t rand_out[RANDOM_NUM_SIZE];
uint8_t device_mac[MAC_SIZE];
uint8_t host_mac[MAC_SIZE];

/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************

void RNG(uint8_t *dest, size_t size)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    uint8_t random[RANDOM_NUM_SIZE];

    if (ATCA_SUCCESS == (status = atcab_random(random)))
    {
        memcpy(dest, random, size);
    }
}

ATCA_STATUS generate_device_mac(void)
{
    ATCA_STATUS status;
    char displaystr[400];
    size_t displaylen;

    RNG(num_in, NONCE_NUMIN_SIZE);
    if (ATCA_SUCCESS == (status = atcab_nonce_rand(num_in, rand_out)))
    {
        if (ATCA_SUCCESS == (status = atcab_mac(MAC_MODE_BLOCK2_TEMPKEY | MAC_MODE_INCLUDE_SN, 
                                                SYMMETRIC_KEY_SLOT, NULL, device_mac)))
        {
            displaylen = sizeof(displaystr);
            atcab_bin2hex(device_mac, MAC_SIZE, displaystr, &displaylen);
            printf("\nMAC received from Accessory device:\r\n%s\r\n", displaystr);
        }
        else
        {
            printf("\nDevice mac calculation is failed with status {%02X}.\r\n", status);
        }
    }
    else
    {
        printf("\nDevice nonce generation is failed with status {%02X}.\r\n", status);
    }

    return status;
}

ATCA_STATUS calculate_host_mac(void)
{
    ATCA_STATUS status;
    atca_temp_key_t temp_key;
    atca_nonce_in_out_t nonce_params;
    atca_mac_in_out_t mac_params;
    uint8_t sernum[ATCA_SERIAL_NUM_SIZE];
    char displaystr[400];
    size_t displaylen;

    // Setup nonce command
    memset(&temp_key, 0, sizeof(temp_key));
    memset(&nonce_params, 0, sizeof(nonce_params));
    nonce_params.mode = NONCE_MODE_SEED_UPDATE;
    nonce_params.zero = 0;
    nonce_params.num_in = num_in;
    nonce_params.rand_out = rand_out;
    nonce_params.temp_key = &temp_key;
    if (ATCA_SUCCESS == (status = atcah_nonce(&nonce_params)))
    {
        if (ATCA_SUCCESS == (status = atcab_read_serial_number(sernum)))
        {
            // Setup MAC command
            memset(&mac_params, 0, sizeof(mac_params));
            mac_params.mode = MAC_MODE_BLOCK2_TEMPKEY | MAC_MODE_INCLUDE_SN;
            mac_params.key_id = SYMMETRIC_KEY_SLOT;
            mac_params.challenge = NULL;
            mac_params.key = slot_5_secret_key;
            mac_params.otp = NULL;
            mac_params.sn = sernum;
            mac_params.response = host_mac;
            mac_params.temp_key = &temp_key;
            if (ATCA_SUCCESS == (status = atcah_mac(&mac_params)))
            {
                displaylen = sizeof(displaystr);
                atcab_bin2hex(device_mac, MAC_SIZE, displaystr, &displaylen);
                printf("\nMAC calculated on Host:\r\n%s\r\n", displaystr);
            }
            else
            {
                printf("\nHost mac calculation is failed with status {%02X}.\r\n", status);
            }
        }
        else
        {
            printf("\nReading serial number is failed with status {%02X}.\r\n", status);
        }
    }
    else
    {
        printf("\nHost nonce generation is failed with status {%02X}.\r\n", status);
    }

    return status;
}

ATCA_STATUS compare_accessory_device_mac_and_host_mac(void)
{
    ATCA_STATUS status;

    if (memcmp(device_mac, host_mac, MAC_SIZE) == 0)
    {
        status = ATCA_SUCCESS;
        printf("\nAccessory device is authenticated successful.\r\n");
    }
    else
    {
        status = ATCA_CHECKMAC_VERIFY_FAILED;
        printf("\nAccessory device is not authenticated\r\n");
    }

    return status;
}


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    ATCA_STATUS status;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
            status = atcab_init(&atecc608_0_init_data);
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_GENERATE_DEVICE_MAC : APP_STATE_FINISH;
            break;

        case APP_STATE_GENERATE_DEVICE_MAC:
            status = generate_device_mac();
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_CALCULATE_HOST_MAC : APP_STATE_FINISH;
            break;

        case APP_STATE_CALCULATE_HOST_MAC:
            status = calculate_host_mac();
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_COMPARE_MAC : APP_STATE_FINISH;
            break;

        case APP_STATE_COMPARE_MAC:
            (void)compare_accessory_device_mac_and_host_mac();
            appData.state = APP_STATE_FINISH;
            break;

        /* TODO: implement your application state machine.*/


        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
